from labgrid.resource.flashrom import Flashrom, NetworkFlashrom
from labgrid.driver.flashromdriver import FlashromDriver


def test_flashrom_network_create(target):
    r = NetworkFlashrom(target,
                       name=None,
                       programmer='linux_spi:dev=/dev/spidev0.1,spispeed=30000',
                       host='localhost')
    assert (isinstance(r, NetworkFlashrom))
    d = FlashromDriver(target, name=None)
    assert (isinstance(d, FlashromDriver))


def test_flashrom_driver_create(target):
    r = Flashrom(target,
                 name=None,
                 programmer='linux_spi:dev=/dev/spidev0.1,spispeed=30000')
    assert (isinstance(r, Flashrom))

    d = FlashromDriver(target, name=None)
    assert (isinstance(d, FlashromDriver))



from labgrid.resource.httpvideostream import HTTPVideoStream
from labgrid.driver.httpvideodriver import HTTPVideoDriver


def test_ipvideo_create(target):
    r = HTTPVideoStream(target, name=None, url="http://localhost/")
    d = HTTPVideoDriver(target, name=None)
    assert isinstance(d, HTTPVideoDriver)



def get_channel_info(driver, channel):
    info = {}
    info['BWLIMIT'] = driver.get_bool(f":CHANnel{channel:d}:BWLimit?")
    info['COUPLING'] = driver.get_str(f":CHANnel{channel:d}:COUPling?").strip()
    info['LABEL'] = driver.get_str(f":CHANnel{channel:d}:LABel?")
    info['PROBE'] = driver.get_decimal(f":CHANnel{channel:d}:PROBe?")
    info['PROBE:HEAD'] = driver.get_str(f":CHANnel{channel:d}:PROBe:HEAD?").strip()
    info['PROBE:ID'] = driver.get_str(f":CHANnel{channel:d}:PROBe:ID?").strip()
    info['PROBE:SKEW'] = driver.get_decimal(f":CHANnel{channel:d}:PROBe:SKEW?")
    info['PROBE:STYPE'] = driver.get_str(f":CHANnel{channel:d}:PROBe:STYPe?").strip()
    info['RANGE'] = driver.get_decimal(f":CHANnel{channel:d}:RANGe?")
    info['SCALE'] = driver.get_decimal(f":CHANnel{channel:d}:SCALe?")
    info['UNITS'] = driver.get_str(f":CHANnel{channel:d}:UNITS?").strip()
    return info

def get_channel_values(driver, channel):
    driver.command(f":MEASURE:SOURCE CHANNEL{channel:d}")
    info = {}
    info['DUTYCYCLE'] = driver.get_decimal(":MEASURE:DUTYcycle?")
    info['FREQUENCY'] = driver.get_decimal(":MEASURE:FREQuency?")
    info['PERIOD'] = driver.get_decimal(":MEASURE:PERiod?")
    info['VMAX'] = driver.get_decimal(":MEASURE:VMAX?")
    info['VMIN'] = driver.get_decimal(":MEASURE:VMIN?")
    info['VPP'] = driver.get_decimal(":MEASURE:VPP?")
    return info

def get_screenshot_png(driver):
    driver.command(":HARDcopy:INKSaver OFF")
    return driver.query(':DISPlay:DATA? PNG, COLor', binary=True)



def test_shell(target):
    ssh_driver = target.get_driver('SSHDriver')
    target.activate(ssh_driver)

    stdout, stderr, returncode = ssh_driver.run('uname -r')

    assert stdout
    print(f'Kernel {stdout}')



import subprocess

import attr

from .common import Driver
from ..factory import target_factory
from ..step import step
from .exception import ExecutionError
from ..util.helper import processwrapper

@target_factory.reg_driver
@attr.s(eq=False)
class USBSDMuxDriver(Driver):
    """The USBSDMuxDriver uses the usbsdmux tool
    (https://github.com/pengutronix/usbsdmux) to control the USB-SD-Mux
    hardware

    Args:
        bindings (dict): driver to use with usbsdmux
    """
    bindings = {
        "mux": {"USBSDMuxDevice", "NetworkUSBSDMuxDevice"},
    }

    def __attrs_post_init__(self):
        super().__attrs_post_init__()
        if self.target.env:
            self.tool = self.target.env.config.get_tool('usbsdmux')
        else:
            self.tool = 'usbsdmux'

    @Driver.check_active
    @step(title='sdmux_set', args=['mode'])
    def set_mode(self, mode):
        if not mode.lower() in ['dut', 'host', 'off', 'client']:
            raise ExecutionError(f"Setting mode '{mode}' not supported by USBSDMuxDriver")
        cmd = self.mux.command_prefix + [
            self.tool,
            self.mux.control_path,
            mode.lower()
        ]
        processwrapper.check_output(cmd)

    @Driver.check_active
    @step(title='sdmux_get')
    def get_mode(self):
        cmd = self.mux.command_prefix + [
            self.tool,
            self.mux.control_path,
            "get"
        ]
        proc = subprocess.run(
            cmd,
            stdout=subprocess.PIPE,
            check=True
        )
        return proc.stdout.strip().decode()



import pexpect

def test_with_feature(tmpdir):
    conf = tmpdir.join("config.yaml")
    conf.write(
"""
targets:
  test1:
    features:
      - test
    drivers: {}
"""
    )
    test = tmpdir.join("test.py")
    test.write(
"""
import pytest

@pytest.mark.lg_feature("test")
def test(env):
    assert True
"""
    )

    with pexpect.spawn(f'pytest --lg-env {conf} {test}') as spawn:
        spawn.expect("1 passed")
        spawn.expect(pexpect.EOF)
        spawn.close()
        assert spawn.exitstatus == 0

def test_skip_feature(tmpdir):
    conf = tmpdir.join("config.yaml")
    conf.write(
"""
targets:
  test1:
    features:
      - test
    drivers: {}
"""
    )
    test = tmpdir.join("test2.py")
    test.write(
"""
import pytest

@pytest.mark.lg_feature("test2")
def test(env):
    assert True
"""
    )

    with pexpect.spawn(f'pytest --lg-env {conf} {test}') as spawn:
        spawn.expect("1 skipped")
        spawn.expect(pexpect.EOF)
        spawn.close()
        assert spawn.exitstatus == 0

def test_skip_feature_list(tmpdir):
    conf = tmpdir.join("config.yaml")
    conf.write(
"""
targets:
  test1:
    features:
      - test
    drivers: {}
"""
    )
    test = tmpdir.join("test2.py")
    test.write(
"""
import pytest

@pytest.mark.lg_feature(["test2", "test3"])
def test(env):
    assert True
"""
    )

    with pexpect.spawn(f'pytest --lg-env {conf} {test}') as spawn:
        spawn.expect("1 skipped")
        spawn.expect(pexpect.EOF)
        spawn.close()
        assert spawn.exitstatus == 0

def test_match_feature_list(tmpdir):
    conf = tmpdir.join("config.yaml")
    conf.write(
"""
targets:
  test1:
    features:
      - test1
      - test2
    drivers: {}
"""
    )
    test = tmpdir.join("test2.py")
    test.write(
"""
import pytest

@pytest.mark.lg_feature(["test1", "test2"])
def test(env):
    assert True
"""
    )

    with pexpect.spawn(f'pytest --lg-env {conf} {test}') as spawn:
        spawn.expect("1 passed")
        spawn.expect(pexpect.EOF)
        spawn.close()
        assert spawn.exitstatus == 0

def test_match_multi_feature_source(tmpdir):
    conf = tmpdir.join("config.yaml")
    conf.write(
"""
targets:
  test1:
    features:
      - test1
      - test2
      - test3
    drivers: {}
"""
    )
    test = tmpdir.join("test.py")
    test.write(
"""
import pytest

pytestmark = pytest.mark.lg_feature("test1")

@pytest.mark.lg_feature("test2")
class TestMulti:
    @pytest.mark.lg_feature("test3")
    def test(self, env):
        assert True
"""
    )

    with pexpect.spawn(f'pytest --lg-env {conf} {test}') as spawn:
        spawn.expect("1 passed")
        spawn.expect(pexpect.EOF)
        spawn.close()
        assert spawn.exitstatus == 0

def test_skip_multi_feature_source(tmpdir):
    conf = tmpdir.join("config.yaml")
    conf.write(
"""
targets:
  test1:
    features:
      - test1
      - test3
    drivers: {}
"""
    )
    test = tmpdir.join("test.py")
    test.write(
"""
import pytest

pytestmark = pytest.mark.lg_feature("test1")

@pytest.mark.lg_feature("test2")
class TestMulti:
    @pytest.mark.lg_feature("test3")
    def test(self, env):
        assert True
"""
    )

    with pexpect.spawn(f'pytest --lg-env {conf} {test}') as spawn:
        spawn.expect("1 skipped")
        spawn.expect(pexpect.EOF)
        spawn.close()
        assert spawn.exitstatus == 0



import enum

import attr

from ..factory import target_factory
from ..step import step
from .common import Strategy, StrategyError


class Status(enum.Enum):
    unknown = 0
    off = 1
    barebox = 2
    shell = 3


@target_factory.reg_driver
@attr.s(eq=False)
class BareboxStrategy(Strategy):
    """BareboxStrategy - Strategy to switch to barebox or shell"""
    bindings = {
        "power": "PowerProtocol",
        "console": "ConsoleProtocol",
        "barebox": "BareboxDriver",
        "shell": "ShellDriver",
    }

    status = attr.ib(default=Status.unknown)

    def __attrs_post_init__(self):
        super().__attrs_post_init__()

    @step(args=['status'])
    def transition(self, status, *, step):  # pylint: disable=redefined-outer-name
        if not isinstance(status, Status):
            status = Status[status]
        if status == Status.unknown:
            raise StrategyError(f"can not transition to {status}")
        elif status == self.status:
            step.skip("nothing to do")
            return  # nothing to do
        elif status == Status.off:
            self.target.deactivate(self.console)
            self.target.activate(self.power)
            self.power.off()
        elif status == Status.barebox:
            self.transition(Status.off)
            self.target.activate(self.console)
            # cycle power
            self.power.cycle()
            # interrupt barebox
            self.target.activate(self.barebox)
        elif status == Status.shell:
            # transition to barebox
            self.transition(Status.barebox)
            self.barebox.boot("")
            self.barebox.await_boot()
            self.target.activate(self.shell)
            self.shell.run("systemctl is-system-running --wait")
        else:
            raise StrategyError(
                f"no transition found from {self.status} to {status}"
            )
        self.status = status

    @step(args=['status'])
    def force(self, status):
        if not isinstance(status, Status):
            status = Status[status]
        self.target.activate(self.power)
        if status == Status.barebox:
            self.target.activate(self.barebox)
        elif status == Status.shell:
            self.target.activate(self.shell)
        else:
            raise StrategyError("can not force state {}".format(status))
        self.status = status



from labgrid.resource.remote import NetworkUSBTMC
from labgrid.driver.usbtmcdriver import USBTMCDriver


def test_usbtmc_create(target):
    r = NetworkUSBTMC(target,
            name=None,
            host="localhost",
            busnum=0,
            devnum=1,
            path='/dev/usbtmc0',
            vendor_id=0x0,
            model_id=0x0,
    )
    d = USBTMCDriver(target, name=None)
    assert (isinstance(d, USBTMCDriver))

def test_import_backends():
    import labgrid.driver.usbtmc.keysight_dsox2000



from labgrid.resource.remote import NetworkSiSPMPowerPort
from labgrid.driver.powerdriver import SiSPMPowerDriver


def test_sispm_create(target):
    r = NetworkSiSPMPowerPort(target,
            name=None,
            host="localhost",
            busnum=0,
            devnum=1,
            path='0:1',
            vendor_id=0x0,
            model_id=0x0,
            index=1,
    )
    d = SiSPMPowerDriver(target, name=None)
    assert (isinstance(d, SiSPMPowerDriver))



import time

from labgrid.driver import ExternalConsoleDriver


class TestExternalConsoleDriver:
    def test_create(self, target):
        d = ExternalConsoleDriver(target, 'console', cmd='cat')
        assert (isinstance(d, ExternalConsoleDriver))

    def test_communicate(self, target):
        data = b"test\ndata"
        d = ExternalConsoleDriver(target, 'console', cmd='cat')
        target.activate(d)
        d.write(data)
        time.sleep(0.1)
        assert d.read(1024) == data  # assert written data is read
        d.write(data)
        time.sleep(0.1)
        assert d.read(5, max_size=5) == data[:5]  # assert max_size limits read bytes
        d.close()



def get_channel_info(driver, channel):
    info = {}
    info['SELECTED'] = driver.get_str(f"SELect:CH{channel:d)}?").strip()
    info['BWLIMIT'] = driver.get_str(f":CH{channel:d}:BANdwidth?").strip()
    info['COUPLING'] = driver.get_str(f":CH{channel:d}:COUPling?").strip()
    info['INVERT'] = driver.get_str(f":CH{channel:d}:INVert?").strip()
    info['POSITION'] = driver.get_decimal(f":CH{channel:d}:POSition?")
    info['PROBE'] = driver.get_decimal(f":CH{channel:d}:PRObe?")
    info['SCALE'] = driver.get_decimal(f":CH{channel:d}:SCAle?")
    return info

def get_channel_values(driver, channel):
    driver.command(f":MEASUrement:IMMed:SOUrce1 CH{channel:d}")
    info = {}
    driver.command(":MEASUrement:IMMed:TYPe MEAN")
    info['MEAN'] = driver.get_decimal(":MEASUrement:IMMed:VALue?")
    driver.command(":MEASUrement:IMMed:TYPe FREQuency")
    info['FREQUENCY'] = driver.get_decimal(":MEASUrement:IMMed:VALue?")
    driver.command(":MEASUrement:IMMed:TYPe PK2pk")
    info['PK2PK'] = driver.get_decimal(":MEASUrement:IMMed:VALue?")
    driver.command(":MEASUrement:IMMed:TYPe MINImum")
    info['MINIMUM'] = driver.get_decimal(":MEASUrement:IMMed:VALue?")
    driver.command(":MEASUrement:IMMed:TYPe MAXImum")
    info['MAXIMUM'] = driver.get_decimal(":MEASUrement:IMMed:VALue?")
    return info

def get_screenshot_tiff(driver):
    driver.command(":hardcopy:format tiff")
    tiff = driver.query(':hardcopy start', raw=True)
    return tiff



from labgrid.resource.mqtt import TasmotaPowerPort
from labgrid.driver.mqtt import TasmotaPowerDriver

import pytest

pytest.importorskip("paho.mqtt.client")


def test_tasmota_resource(target, mocker):
    mocker.patch('paho.mqtt.client.Client.connect', return_value=None)
    mocker.patch('paho.mqtt.client.Client.loop_start', return_value=None)
    TasmotaPowerPort(target, name=None, host="localhost", avail_topic="test",
                     power_topic="test", status_topic="test")


def test_tasmota_driver(target, mocker):
    mocker.patch('paho.mqtt.client.Client.connect', return_value=None)
    mocker.patch('paho.mqtt.client.Client.loop_start', return_value=None)
    res = TasmotaPowerPort(target, name=None, host="localhost", avail_topic="test",
                           power_topic="test", status_topic="test")
    res.manager._available.add("test")
    driver = TasmotaPowerDriver(target, name=None)
    target.activate(driver)



import pytest


@pytest.fixture
def graph_strategy(target):
    from labgrid.strategy import GraphStrategy

    class TestStrategy(GraphStrategy):
        def state_Root(self):
            pass

        @GraphStrategy.depends('Root')
        def state_A1(self):
            pass

        @GraphStrategy.depends('Root')
        def state_A2(self):
            pass

        @GraphStrategy.depends('A1', 'A2')
        def state_B(self):
            pass

        @GraphStrategy.depends('B')
        def state_C1(self):
            pass

        @GraphStrategy.depends('B')
        def state_C2(self):
            pass

        @GraphStrategy.depends('C1', 'C2')
        def state_D(self):
            pass

    return TestStrategy(target, 'strategy')


# api tests ###################################################################
@pytest.mark.dependency
def test_fixture(graph_strategy, target):
    # test root state
    assert graph_strategy.root_state == 'Root'

    # test path
    assert graph_strategy.path == []

    # test state discovering
    state_names = sorted(list(graph_strategy.states.keys()))
    expected_state_names = ['A1', 'A2', 'B', 'C1', 'C2', 'D', 'Root']

    assert state_names == expected_state_names


@pytest.mark.dependency(depends=['test_fixture'])
def test_strategy_without_states(target):
    from labgrid.strategy import GraphStrategy, InvalidGraphStrategyError

    class TestStrategy(GraphStrategy):
        pass

    with pytest.raises(InvalidGraphStrategyError) as e:
        TestStrategy(target, 'strategy')

    assert e.value.msg == 'GraphStrategies without states are invalid'


@pytest.mark.dependency(depends=['test_fixture'])
def test_strategy_without_root_state(target):
    from labgrid.strategy import GraphStrategy, InvalidGraphStrategyError

    class TestStrategy(GraphStrategy):
        @GraphStrategy.depends('B')
        def state_A(self):
            pass

        @GraphStrategy.depends('A')
        def state_B(self):
            pass

    with pytest.raises(InvalidGraphStrategyError) as e:
        TestStrategy(target, 'strategy')

    assert e.value.msg == 'GraphStrategies without root state are invalid'


@pytest.mark.dependency(depends=['test_fixture'])
def test_multiple_root_states(target):
    from labgrid.strategy import GraphStrategy, InvalidGraphStrategyError

    class TestStrategy(GraphStrategy):
        def state_Root(self):
            pass

        def state_Root2(self):
            pass

    with pytest.raises(InvalidGraphStrategyError) as e:
        TestStrategy(target, 'strategy')

    assert e.value.msg.startswith("Only one root state supported.")


@pytest.mark.dependency(depends=['test_fixture'])
def test_unknown_dependencies(target):
    from labgrid.strategy import GraphStrategy, InvalidGraphStrategyError

    class TestStrategy(GraphStrategy):
        def state_Root(self):
            pass

        @GraphStrategy.depends('Root')
        def state_A(self):
            pass

        @GraphStrategy.depends('Root', 'A', 'C')
        def state_B(self):
            pass

    with pytest.raises(InvalidGraphStrategyError) as e:
        TestStrategy(target, 'strategy')

    assert e.value.msg.startswith("B: State 'C' is unknown")


@pytest.mark.dependency(depends=['test_fixture'])
def test_strategy_with_uncallable_states(target):
    from labgrid.strategy import GraphStrategy, InvalidGraphStrategyError

    class TestStrategy(GraphStrategy):
        state_foo = 1

        def state_Root(self):
            pass

    with pytest.raises(InvalidGraphStrategyError) as e:
        TestStrategy(target, 'strategy')

    assert e.value.msg.startswith(
        "GraphStrategy state 'state_foo' is not callable")


@pytest.mark.dependency(name='api-works',
                        depends=[
                            'test_fixture',
                            'test_strategy_without_states',
                            'test_strategy_without_root_state',
                            'test_unknown_dependencies',
                            'test_strategy_with_uncallable_states',
                        ])
def test_api_works():
    pass


# functional tests ############################################################
@pytest.mark.dependency(depends=['api-works'])
def test_graphviz_graph(graph_strategy):
    pytest.importorskip("graphviz")
    graph_strategy.graph
    graph_strategy.graph  # trigger the caching branch in the graph code

    graph_strategy.path = ['Root', 'A1', 'B']  # fake a post transition graph
    graph_strategy.graph


@pytest.mark.dependency(depends=['api-works'])
def test_transition(graph_strategy):
    assert graph_strategy.transition('B') == ['Root', 'A1', 'B']
    assert graph_strategy.path == ['Root', 'A1', 'B']

    assert graph_strategy.transition('B') == []
    assert graph_strategy.path == ['Root', 'A1', 'B']


@pytest.mark.dependency(depends=['api-works', 'test_transition'])
def test_transition_to_unknown_state(graph_strategy):
    from labgrid.strategy import GraphStrategyRuntimeError

    with pytest.raises(GraphStrategyRuntimeError) as e:
        graph_strategy.transition('G')

    assert e.value.msg.startswith("Unknown state 'G'.")


@pytest.mark.dependency(depends=['api-works', 'test_transition'])
def test_interleaved_transitions(target):
    from labgrid.strategy import GraphStrategy, GraphStrategyRuntimeError

    class TestStrategy(GraphStrategy):
        def state_Root(self):
            pass

        @GraphStrategy.depends('Root')
        def state_A(self):
            self.transition('B')

        @GraphStrategy.depends('Root')
        def state_B(self):
            self.transition('A')

    strategy = TestStrategy(target, 'strategy')

    with pytest.raises(GraphStrategyRuntimeError) as e:
        strategy.transition('A')

    assert e.value.msg == 'Another transition is already running'


@pytest.mark.dependency(depends=['api-works', 'test_transition'])
def test_transition_via(graph_strategy):
    assert graph_strategy.transition('D', via=['A2']) == ['Root', 'A2', 'B', 'C1', 'D']
    assert graph_strategy.path == ['Root', 'A2', 'B', 'C1', 'D']

    graph_strategy.invalidate()

    assert graph_strategy.transition('D:A2') == ['Root', 'A2', 'B', 'C1', 'D']
    assert graph_strategy.path == ['Root', 'A2', 'B', 'C1', 'D']

    assert graph_strategy.transition('D:C2,A2') == ['Root', 'A2', 'B', 'C2', 'D']
    assert graph_strategy.path == ['Root', 'A2', 'B', 'C2', 'D']

@pytest.mark.dependency(depends=['api-works',
                                 'test_transition',
                                 'test_transition_via'])
def test_incremental_transition(graph_strategy):
    assert graph_strategy.transition('B') == ['Root', 'A1', 'B']
    assert graph_strategy.transition('D') == ['C1', 'D']

    graph_strategy.invalidate()

    assert graph_strategy.transition('B', via=['A2']) == ['Root', 'A2', 'B']
    assert graph_strategy.transition('D') == ['Root', 'A1', 'B', 'C1', 'D']


@pytest.mark.dependency(depends=['api-works', 'test_transition'])
def test_transition_error(target):
    from labgrid.strategy import GraphStrategy

    class TestStrategy(GraphStrategy):
        def state_Root(self):
            pass

        @GraphStrategy.depends('Root')
        def state_A(self):
            pass

        @GraphStrategy.depends('A')
        def state_B(self):
            raise Exception

    strategy = TestStrategy(target, 'strategy')

    strategy.transition('A')
    assert strategy.path == ['Root', 'A']

    with pytest.raises(Exception):
        strategy.transition('B')

    with pytest.raises(Exception):
        strategy.transition('B', via='B')

    assert strategy.path == []



import attr


@attr.s(eq=False)
class ExecutionError(Exception):
    msg = attr.ib(validator=attr.validators.instance_of(str))
    stdout = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.instance_of(list))
    )
    stderr = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.instance_of(list))
    )


@attr.s(eq=False)
class CleanUpError(Exception):
    msg = attr.ib(validator=attr.validators.instance_of(str))



import pytest
from py.path import local

import labgrid.util.agentwrapper
from labgrid.util.agentwrapper import AgentError, AgentException, AgentWrapper, b2s, s2b

@pytest.fixture(scope='function')
def subprocess_mock(mocker):
    import subprocess

    original = subprocess.Popen

    agent = local(labgrid.util.agentwrapper.__file__).dirpath('agent.py')

    def run(args, **kwargs):
        assert args[0] in ['rsync', 'ssh']
        if args[0] == 'rsync':
            src = local(args[-2])
            assert src == agent
            dst = args[-1]
            assert ':' in dst
            dst = dst.split(':', 1)[1]
            assert '/' not in dst
            assert dst.startswith('.labgrid_agent')
            return original(['true'], **kwargs)
        elif args[0] == 'ssh':
            assert '--' in args
            args = args[args.index('--')+1:]
            assert len(args) == 2
            assert args[0] == 'python3'
            assert args[1].startswith('.labgrid_agent')
            # we need to use the original here to get the coverage right
            return original(['python3', str(agent)], **kwargs)

    mocker.patch('subprocess.Popen', run)

def test_create(subprocess_mock):
    aw = AgentWrapper('localhost')
    aw.close()

def test_call(subprocess_mock):
    aw = AgentWrapper('localhost')
    assert aw.call('test') == []
    assert aw.call('test', 0) == [0]
    assert aw.call('test', 0, 1) == [1, 0]
    assert aw.call('test', 'foo') == ['foo']
    assert aw.call('test', '{') == ['{']

def test_proxy(subprocess_mock):
    aw = AgentWrapper('localhost')
    assert aw.test() == []
    assert aw.test( 0, 1) == [1, 0]

def test_bytes(subprocess_mock):
    aw = AgentWrapper('localhost')
    assert s2b(aw.test(b2s(b'\x00foo'))[0]) == b'\x00foo'

def test_exception(subprocess_mock):
    aw = AgentWrapper('localhost')

    with pytest.raises(AgentException) as excinfo:
        aw.error('foo')
    assert excinfo.value.args == ("ValueError('foo')",)

    with pytest.raises(AgentException) as excinfo:
        aw.error('bar')
    assert excinfo.value.args == ("ValueError('bar')",)

def test_error(subprocess_mock):
    aw = AgentWrapper('localhost')
    aw.agent.stdin.write(b'\x00')
    with pytest.raises(AgentError):
        aw.test()

def test_module(subprocess_mock):
    aw = AgentWrapper('localhost')
    dummy = aw.load('dummy')
    assert dummy.neg(1) == -1

def test_local():
    aw = AgentWrapper(None)

    assert aw.test() == []
    assert aw.test( 0, 1) == [1, 0]

    assert s2b(aw.test(b2s(b'\x00foo'))[0]) == b'\x00foo'

    with pytest.raises(AgentException) as excinfo:
        aw.error('foo')
    assert excinfo.value.args == ("ValueError('foo')",)

    with pytest.raises(AgentException) as excinfo:
        aw.error('bar')
    assert excinfo.value.args == ("ValueError('bar')",)

    dummy = aw.load('dummy')
    assert dummy.neg(1) == -1

def test_all_modules():
    aw = AgentWrapper(None)
    aw.load('deditec_relais8')
    methods = aw.list()
    assert 'deditec_relais8.set' in methods
    assert 'deditec_relais8.get' in methods
    aw.load('sysfsgpio')
    methods = aw.list()
    assert 'sysfsgpio.set' in methods
    assert 'sysfsgpio.get' in methods
    aw.load('usb_hid_relay')
    methods = aw.list()
    assert 'usb_hid_relay.set' in methods
    assert 'usb_hid_relay.get' in methods

def test_import_modules():
    import labgrid.util.agents
    import labgrid.util.agents.dummy
    from labgrid.util.agents import deditec_relais8, sysfsgpio



import attr

from ..factory import target_factory
from .common import NetworkResource, Resource


@target_factory.reg_resource
@attr.s(eq=False)
class DediprogFlasher(Resource):
    vcc = attr.ib(validator=attr.validators.in_(('3.5V', '2.5V', '1.8V')))

@target_factory.reg_resource
@attr.s(eq=False)
class NetworkDediprogFlasher(NetworkResource):
    vcc = attr.ib(validator=attr.validators.in_(('3.5V', '2.5V', '1.8V')))



from labgrid.resource.pyvisa import PyVISADevice
from labgrid.driver.pyvisadriver import PyVISADriver

import pytest

pytest.importorskip("pyvisa")

def test_pyvisa_resource(target):
    PyVISADevice(target, name=None, type='TCPIP', url='127.0.0.1')


def test_resource_driver(target, mocker):
    PyVISADevice(target, name=None, type='TCPIP', url='127.0.0.1')
    driver = PyVISADriver(target, name=None)

    mocker.patch('pyvisa.ResourceManager.open_resource', return_value=None)
    target.activate(driver)



from labgrid.resource.remote import NetworkUSBVideo
from labgrid.driver.usbvideodriver import USBVideoDriver


def test_usbvideo_create(target):
    r = NetworkUSBVideo(target,
            name=None,
            host="localhost",
            busnum=0,
            devnum=1,
            path='0:1',
            vendor_id=0x0,
            model_id=0x0,
    )
    d = USBVideoDriver(target, name=None)
    assert (isinstance(d, USBVideoDriver))




"""Rest interface for controlling power port, using PUT / GET on a URL.

  NetworkPowerPort:
      model: rest
      host: 'http://192.168.0.42/relay/{index}/value'
      index: 3

  Will do a GET request to http://192.168.0.42/relay/3/value to get current
  relay state, expecting a response of either '0' (relay off) or '1' (relay
  on), and a PUT request to http://192.168.0.42/relay/3/value with request
  data of '0' or '1' to change relay state.

"""

import requests

def power_set(host, port, index, value):
    assert port is None
    value = b"1" if value else b"0"
    r = requests.put(host.format(index=index), data=value)
    r.raise_for_status()

def power_get(host, port, index):
    assert port is None
    r = requests.get(host.format(index=index))
    r.raise_for_status()
    return r.text == "1"



import pytest

from labgrid.driver import FileDigitalOutputDriver

def test_filedigital_instance(target, mocker):
    m_isfile = mocker.patch('os.path.isfile', return_value=True)
    d = FileDigitalOutputDriver(target, name=None, filepath='/dev/null')
    target.activate(d)
    assert isinstance(d, FileDigitalOutputDriver)
    m_isfile.assert_called_once_with('/dev/null')

def test_filedigital_set(target, mocker):
    m_isfile = mocker.patch('os.path.isfile', return_value=True)
    m_open = mocker.patch('builtins.open', mocker.mock_open())
    d = FileDigitalOutputDriver(target, name=None, filepath='/dev/null')
    target.activate(d)
    d.set(True)
    m_isfile.assert_called_once_with('/dev/null')
    m_open.assert_called_once_with('/dev/null', 'w')
    m_open.return_value.write.assert_called_once_with('1\n')

def test_filedigital_get(target, mocker):
    m_isfile = mocker.patch('os.path.isfile', return_value=True)
    m_open = mocker.patch('builtins.open', mocker.mock_open(read_data='1\n'))
    d = FileDigitalOutputDriver(target, name=None, filepath='/dev/null')
    target.activate(d)
    assert d.get() is True
    m_isfile.assert_called_once_with('/dev/null')
    m_open.assert_called_once_with('/dev/null')
    m_open.return_value.read.assert_called_once()



from labgrid.remote.scheduler import *

def test_simple():
    places = [
        TagSet('place-1', {'name=place-1', 'soc=mx6'}),
        TagSet('place-2', {'name=place-2', 'soc=mx8'}),
    ]
    filters = [
        TagSet('res-1', {'soc=mx8'}),
        TagSet('res-2', {'soc=mx6'}),
    ]
    expected = {
        'res-1': 'place-2',
        'res-2': 'place-1',
    }

    assert schedule(places, filters) == expected
    assert schedule(places, filters[::-1]) == expected
    assert schedule(places[::-1], filters) == expected
    assert schedule(places[::-1], filters[::-1]) == expected


def test_overlap():
    places = [
        TagSet('place-1', {'name=place-1', 'soc=mx6', 'arch=arm'}),
        TagSet('place-2', {'name=place-2', 'soc=mx8', 'arch=arm'}),
    ]
    filters = [
        TagSet('res-1', {'arch=arm'}),
        TagSet('res-2', {'soc=mx6'}),
    ]
    expected = {
        'res-1': 'place-2',
        'res-2': 'place-1',
    }

    assert schedule(places, filters) == expected
    assert schedule(places, filters[::-1]) == expected
    assert schedule(places[::-1], filters) == expected
    assert schedule(places[::-1], filters[::-1]) == expected


def test_flexible():
    places = [
        TagSet('place-1', {'name=place-1', 'board=foo'}),
        TagSet('place-2', {'name=place-2', 'board=foo'}),
    ]
    filters = [
        TagSet('res-1', {'board=foo'}),
        TagSet('res-2', {'board=foo'}),
    ]
    expected_keys = {'res-1', 'res-2'}
    expected_vals = {'place-1', 'place-2'}

    allocation = schedule(places, filters)
    assert allocation.keys() == expected_keys
    assert set(allocation.values()) == expected_vals
    allocation = schedule(places, filters[::-1])
    assert allocation.keys() == expected_keys
    assert set(allocation.values()) == expected_vals
    allocation = schedule(places[::-1], filters)
    assert allocation.keys() == expected_keys
    assert set(allocation.values()) == expected_vals
    allocation = schedule(places[::-1], filters[::-1])
    assert allocation.keys() == expected_keys
    assert set(allocation.values()) == expected_vals


def test_conflict():
    places = [
        TagSet('place-1', {'name=place-1', 'board=foo'}),
    ]
    filters = [
        TagSet('res-1', {'board=foo'}),
        TagSet('res-2', {'board=foo'}),
    ]

    assert schedule(places, filters) == {'res-1': 'place-1'}
    assert schedule(places, filters[::-1]) == {'res-2': 'place-1'}
    assert schedule(places[::-1], filters) == {'res-1': 'place-1'}
    assert schedule(places[::-1], filters[::-1]) == {'res-2': 'place-1'}



import attr

from .common import Driver
from ..factory import target_factory
from ..step import step
from .exception import ExecutionError
from ..util.helper import processwrapper

@target_factory.reg_driver
@attr.s(eq=False)
class LXAUSBMuxDriver(Driver):
    """The LXAUSBMuxDriver uses the usbmuxctl tool to control the USBMux
    hardware
    """
    bindings = {
        "mux": {"LXAUSBMux", "NetworkLXAUSBMux"},
    }

    def __attrs_post_init__(self):
        super().__attrs_post_init__()
        if self.target.env:
            self.tool = self.target.env.config.get_tool('usbmuxctl')
        else:
            self.tool = 'usbmuxctl'

    @Driver.check_active
    @step(title='usbmux_set', args=['links'])
    def set_links(self, links):
        args = []
        for link in links:
            link = link.lower()
            if link == 'dut-device':
                args.append('--dut-device')
            elif link == 'host-dut':
                args.append('--host-dut')
            elif link == 'host-device':
                args.append('--host-device')
            else:
                raise ExecutionError(f"Link '{link}' not supported by LXAUSBMuxDriver")

        cmd = self.mux.command_prefix + [
            self.tool,
            "--path",
            self.mux.path,
            "connect",
            *args,
        ]
        processwrapper.check_output(cmd)



"""Interface for controlling relays of Shelly devices using the Gen 1 API

  NetworkPowerPort:
      model: shelly_gen1
      host: 'http://192.168.0.42'
      index: 0

  Will do a GET request to http://192.168.0.42/relay/0 to get the current
  relay state, and a POST request to http://192.168.0.42/relay/0 with request
  data of 'turn=off' or 'turn=on' to change the relay state.

  Also, see the official Gen 1 Device API documentation:
  https://shelly-api-docs.shelly.cloud/gen1/

"""
import json

import requests

def power_set(host:str, port:int, index:int=0, value:bool=True):
    assert not port
    turn = "on" if value else "off"
    r = requests.post(f"{host}/relay/{index}", data={'turn': turn})
    r.raise_for_status()

def power_get(host:str, port:int, index:int):
    assert not port
    r = requests.get(f"{host}/relay/{index}")
    r.raise_for_status()
    return json.loads(r.text)['ison']



""" Simple rest interface for Power Port. Used for ex. misc Raspberry Pi configs
    Author: Kjeld Flarup <kfa@deif.com>

    The URL given in hosts in exporter.yaml must replace {value} with '0' or '1'
    It is optional whether to use {index} or not.

  NetworkPowerPort:
      model: simplerest
      host: 'http://172.17.180.53:9999/relay/{index}/{value}'
      index: 0
"""

import requests

def power_set(host, port, index, value):
    assert port is None

    index = int(index)
    value = 1 if value else 0
    r = requests.get(host.format(value=value, index=index))
    r.raise_for_status()

def power_get(host, port, index):
    assert port is None

    index = int(index)
    # remove trailing /
    r = requests.get(host.format(value='', index=index).rstrip('/'))
    r.raise_for_status()
    return r.text == '1'
